//
//  CubeView.m
//  Cube
//
//  Created by Chris Osborn on 7/1/15.
//  Copyright (c) 2015 Google. All rights reserved.
//

#import "CubeView.h"
#import "CubeViewShader.h"

typedef struct {
    RIPoint3F position;
    RIPoint3F normal;
} CubeVertex;

static const CubeVertex kCubeVertices[] = {
    {{0.0f, 0.0f, 0.0f}, {-1.0f,  0.0f,  0.0f}},
    {{0.0f, 0.0f, 1.0f}, {-1.0f,  0.0f,  0.0f}},
    {{0.0f, 1.0f, 1.0f}, {-1.0f,  0.0f,  0.0f}},
    {{1.0f, 1.0f, 0.0f}, { 0.0f,  0.0f, -1.0f}},
    {{0.0f, 0.0f, 0.0f}, { 0.0f,  0.0f, -1.0f}},
    {{0.0f, 1.0f, 0.0f}, { 0.0f,  0.0f, -1.0f}},
    {{1.0f, 0.0f, 1.0f}, { 0.0f, -1.0f,  0.0f}},
    {{0.0f, 0.0f, 0.0f}, { 0.0f, -1.0f,  0.0f}},
    {{1.0f, 0.0f, 0.0f}, { 0.0f, -1.0f,  0.0f}},
    {{1.0f, 1.0f, 0.0f}, { 0.0f,  0.0f, -1.0f}},
    {{1.0f, 0.0f, 0.0f}, { 0.0f,  0.0f, -1.0f}},
    {{0.0f, 0.0f, 0.0f}, { 0.0f,  0.0f, -1.0f}},
    {{0.0f, 0.0f, 0.0f}, {-1.0f,  0.0f,  0.0f}},
    {{0.0f, 1.0f, 1.0f}, {-1.0f,  0.0f,  0.0f}},
    {{0.0f, 1.0f, 0.0f}, {-1.0f,  0.0f,  0.0f}},
    {{1.0f, 0.0f, 1.0f}, { 0.0f, -1.0f,  0.0f}},
    {{0.0f, 0.0f, 1.0f}, { 0.0f, -1.0f,  0.0f}},
    {{0.0f, 0.0f, 0.0f}, { 0.0f, -1.0f,  0.0f}},
    {{0.0f, 1.0f, 1.0f}, { 0.0f,  0.0f,  1.0f}},
    {{0.0f, 0.0f, 1.0f}, { 0.0f,  0.0f,  1.0f}},
    {{1.0f, 0.0f, 1.0f}, { 0.0f,  0.0f,  1.0f}},
    {{1.0f, 1.0f, 1.0f}, { 1.0f,  0.0f,  0.0f}},
    {{1.0f, 0.0f, 0.0f}, { 1.0f,  0.0f,  0.0f}},
    {{1.0f, 1.0f, 0.0f}, { 1.0f,  0.0f,  0.0f}},
    {{1.0f, 0.0f, 0.0f}, { 1.0f,  0.0f,  0.0f}},
    {{1.0f, 1.0f, 1.0f}, { 1.0f,  0.0f,  0.0f}},
    {{1.0f, 0.0f, 1.0f}, { 1.0f,  0.0f,  0.0f}},
    {{1.0f, 1.0f, 1.0f}, { 0.0f,  1.0f,  0.0f}},
    {{1.0f, 1.0f, 0.0f}, { 0.0f,  1.0f,  0.0f}},
    {{0.0f, 1.0f, 0.0f}, { 0.0f,  1.0f,  0.0f}},
    {{1.0f, 1.0f, 1.0f}, { 0.0f,  1.0f,  0.0f}},
    {{0.0f, 1.0f, 0.0f}, { 0.0f,  1.0f,  0.0f}},
    {{0.0f, 1.0f, 1.0f}, { 0.0f,  1.0f,  0.0f}},
    {{1.0f, 1.0f, 1.0f}, { 0.0f,  0.0f,  1.0f}},
    {{0.0f, 1.0f, 1.0f}, { 0.0f,  0.0f,  1.0f}},
    {{1.0f, 0.0f, 1.0f}, { 0.0f,  0.0f,  1.0f}},
};

@implementation CubeView {
    GLuint _vertexArray;
    GLuint _vertexBuffer;
    CubeViewShader *_shader;
}

- (instancetype)init {
    if ((self = [super init]) != nil) {
        // Properties
        _color = RIColorPurpleRGBA;
        _lightDirection = RIPointMake3F(0.0f, 0.0f, 1.0f);
        
        // Vertices
        [[RIRenderContext activeContext] dispatchBlock:^{
            glGenVertexArrays(1, &_vertexArray);
            glBindVertexArray(_vertexArray);
            
            glGenBuffers(1, &_vertexBuffer);
            glBindBuffer(GL_ARRAY_BUFFER, _vertexBuffer);
            glBufferData(GL_ARRAY_BUFFER, sizeof(kCubeVertices), kCubeVertices, GL_STATIC_DRAW);
            
            glEnableVertexAttribArray(RIShaderAttributePosition);
            glVertexAttribPointer(RIShaderAttributePosition, 3,
                                  GL_FLOAT, GL_FALSE,
                                  sizeof(CubeVertex),
                                  (const GLvoid *)offsetof(CubeVertex, position));
            
            glEnableVertexAttribArray(RIShaderAttributeNormal);
            glVertexAttribPointer(RIShaderAttributeNormal, 3,
                                  GL_FLOAT, GL_FALSE,
                                  sizeof(CubeVertex),
                                  (const GLvoid *)offsetof(CubeVertex, normal));
            
            glBindVertexArray(0);
        }];
        
        // Shader
        _shader = (CubeViewShader *)[[RIShaderCache sharedCache] shaderForKey:CubeViewShaderKey];
        if (_shader == nil) {
            _shader = [[CubeViewShader alloc] init];
            [[RIShaderCache sharedCache] setShader:_shader forKey:CubeViewShaderKey];
        }
    }
    return self;
}

- (void)dealloc {
    GLuint vertexArray = _vertexArray;
    GLuint vertexBuffer = _vertexBuffer;
    
    [[RIRenderContext activeContext] dispatchBlock:^{
        if (vertexArray != 0) {
            glBindVertexArray(0);
            glDeleteVertexArrays(1, &vertexArray);
        }
        if (vertexBuffer != 0) {
            glBindBuffer(GL_ARRAY_BUFFER, 0);
            glDeleteBuffers(1, &vertexBuffer);
        }
    }];
}

#pragma mark -
#pragma mark - Properties

- (void)setColor:(RIColorRGBA)color {
    if (RIColorIsEqualToColorRGBA(color, _color))
        return;
    
    _color = color;
    
    [self setNeedsDraw];
}

- (void)setLightDirection:(RIPoint3F)lightDirection {
    lightDirection = RIPointNormalize3F(lightDirection);
    if (RIPointIsEqualToPoint3F(lightDirection, _lightDirection))
        return;
    
    _lightDirection = lightDirection;
    
    [self setNeedsDraw];
}

#pragma mark -
#pragma mark Drawing

- (void)draw {
    RIColorRGBA effectiveColor = _color;
    effectiveColor.alpha *= self.effectiveOpacity;
    
    [_shader use];
    _shader.normalMatrix = GLKMatrix3Transpose(GLKMatrix4GetMatrix3(self.inverseTransformMatrix));
    _shader.size = self.size3;
    _shader.color = effectiveColor;
    _shader.lightDirection = _lightDirection;
    
    [[RIRenderContext activeContext] dispatchBlock:^{
        glEnable(GL_DEPTH_TEST);
        if (effectiveColor.alpha < 1.0f) {
            glEnable(GL_BLEND);
            glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
        } else {
            glDisable(GL_BLEND);
        }
        glBindVertexArray(_vertexArray);
        glDrawArrays(GL_TRIANGLES, 0, sizeof(kCubeVertices)/sizeof(kCubeVertices[0]));
        glBindVertexArray(0);
    }];
}

@end
